@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: dialog.c,v 1.7 1999/01/13 21:08:41 kahn Exp kahn $
+ * $Id: dialog.c,v 1.20 2000/02/21 18:31:26 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -23,10 +23,33 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include "main.h"
+#ifdef USE_GNOME
+#include <gnome.h>
+#endif
 #include "misc.h"
 #include "dialog.h"
+#include "gnpintl.h"
+
+#ifdef USE_GNOME
+/*
+ * PUBLIC: do_dialog_error()
+ *
+ * wrapper for 'Ok' error dialog box
+ */
+int
+do_dialog_error(char *title, char *msg)
+{
+	GtkWidget *dlg;
+
+	dlg = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_ERROR,
+				    GNOME_STOCK_BUTTON_OK, NULL);
+
+	return (1 + gnome_dialog_run_and_close(GNOME_DIALOG(dlg)));
+} /* do_dialog_error */
+#endif	/* no gnome */
 
 
+/*** past here, these are used for both gnome and non-gnome ***/
 /*** local types ***/
 typedef struct dlg_data {
 	GtkWidget *     dlg;	/* top level dialog widget */
@@ -89,24 +112,30 @@ do_dialog(char *title, char *msg, short num, char **buttons, short dflt,
 		return -1;
 
 	dlg = gtk_dialog_new();
-	
-	gtk_window_set_title(GTK_WINDOW(dlg), (title) ? title : "Dialog Title");
 
-	tmp = gtk_label_new((msg) ? msg : "Hello world!");
+	gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);
+	gtk_window_set_title(GTK_WINDOW(dlg), (title) ? title : _("Dialog Title"));
+	gtk_widget_set_name(dlg, "gnp+");
+	gtk_window_set_wmclass(GTK_WINDOW(dlg), "gnp+", "gnp+");
+	gtk_window_set_policy(GTK_WINDOW(dlg), TRUE, TRUE, FALSE);
+
+	tmp = gtk_label_new((msg) ? msg : ((title) ? title : _("Hello world!")));
 	gtk_box_pack_start(
 		GTK_BOX(GTK_DIALOG(dlg)->vbox), tmp, TRUE, FALSE, 10);
 	gtk_widget_show(tmp);
 
-	wgts = (GtkWidget **)g_malloc(num * sizeof(GtkWidget *));
-	dlgdata = (dlg_data_t *)g_malloc0(sizeof(dlg_data_t));
+	wgts = g_new(GtkWidget *, num);
+	dlgdata = g_new0(dlg_data_t, 1);
 	dlgdata->dlg = dlg;
-	dlgdata->wgts = (GtkWidget **)g_malloc0(num * sizeof(GtkWidget *));
+	dlgdata->wgts = g_new0(GtkWidget *, num);
 	dlgdata->num = num;
 	dlgdata->id = -1;
 	dlgdata->cbs = cbs;
 	dlgdata->cbd = cbd;
 
 	for (i = 0; i < num; i++) {
+		int opts = PACK_START | PACK_EXPAND | PACK_FILL | SHOW_BUTTON;
+#if 0
 		tmp = gtk_button_new_with_label(buttons[i]);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area),
 			tmp, TRUE, TRUE, 2);
@@ -115,19 +144,37 @@ do_dialog(char *title, char *msg, short num, char **buttons, short dflt,
 
 		cbfunc = (cbs && cbs[i]) ? cbs[i] : (GtkSignalFunc *)def_cb;
 		cbdata = (cbd && cbd[i]) ? (char *)(cbd[i]) : (char *)dlgdata;
-		gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-			GTK_SIGNAL_FUNC(cbfunc), cbdata);
+		(void)gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
+					 GTK_SIGNAL_FUNC(cbfunc), cbdata);
 		if (cbfunc != (GtkSignalFunc *)def_cb) {
-			gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-				GTK_SIGNAL_FUNC(def_cb), dlgdata);
+			(void)gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
+						 GTK_SIGNAL_FUNC(def_cb),
+						 dlgdata);
 		}
-
 		gtk_widget_show(tmp);
 
 		if (dflt == i + 1) {
 			GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
 			gtk_widget_grab_default(tmp);
 		}
+#endif
+		if (dflt == i + 1)
+			opts |= (CANCEL_DEFAULT | GRAB_DEFAULT);
+
+		cbfunc = (cbs && cbs[i]) ? cbs[i] : (GtkSignalFunc *)def_cb;
+		cbdata = (cbd && cbd[i]) ? (char *)(cbd[i]) : (char *)dlgdata;
+
+		dlgdata->wgts[i] = misc_button_new_w_label(
+						buttons[i], NULL,
+						GTK_SIGNAL_FUNC(cbfunc), cbdata,
+						GTK_DIALOG(dlg)->action_area,
+						opts, 2);
+		if (cbfunc != (GtkSignalFunc *)def_cb) {
+			(void)gtk_signal_connect(GTK_OBJECT(dlgdata->wgts[i]),
+						 "clicked",
+						 GTK_SIGNAL_FUNC(def_cb),
+						 dlgdata);
+		}
 	}
 
 	gtk_widget_show(dlg);
@@ -149,35 +196,42 @@ do_dialog(char *title, char *msg, short num, char **buttons, short dflt,
 } /* do_dialog */
 
 
+/*** local function definitions ***/
 /*
- * PUBLIC: do_dialog_yes_no_cancel()
+ * PRIVATE: def_cb
  *
- * wrapper for yes-no-cancel dialog box
+ * callback for all button widgets used in do_dialog().  upon invocation,
+ * the callback data contains an array of the widget pointers created in
+ * do_dialog().  this array is scanned and if a match is found, the id num
+ * in the callback data is set, the top-level dialog widget is destroyed,
+ * and the dialog widget is set to NULL.  setting the dialog widget to NULL
+ * will cause the while loop in do_dialog() to exit.
  */
-int
-do_dialog_yes_no_cancel(char *title, char *msg)
+static void
+def_cb(GtkWidget *wgt, gpointer cbdata)
 {
-	char *buttons[] = { BUTTON_YES, BUTTON_NO, BUTTON_CANCEL };
+	short i;
+	dlg_data_t *data = (dlg_data_t *)cbdata;
 
-	return do_dialog(title, msg, 3, buttons, 3, NULL, NULL, TRUE);
-} /* do_dialog_yes_no_cancel */
+	g_assert(data != NULL);
+	g_assert(data->num > 0);
+	g_assert(data->wgts != NULL);
+	g_assert(data->id == -1);
 
+	for (i = 0; i < data->num; i++) {
+		if (data->wgts[i] == wgt) {
+			data->id = i;
+			break;
+		}
+	}
 
-/*
- * PUBLIC: do_dialog_ok()
- *
- * wrapper for 'Ok' dialog box
- */
-int
-do_dialog_ok(char *title, char *msg)
-{
-	char *buttons[] = { BUTTON_OK };
-
-	return do_dialog(title, msg, 1, buttons, 1, NULL, NULL, TRUE);
-} /* do_dialog_ok */
+	gtk_widget_destroy(data->dlg);
+	data->dlg = NULL;	/* this exits the while loop in do_dialog() */
+} /* def_cb */
 
 
 /*
+ * GNOME: gnome_request_dialog()
  * PUBLIC: get_dialog_entry
  *
  * creates a basic, non-modal entry dialog with two buttons: "Ok" and "Cancel".
@@ -221,8 +275,9 @@ get_dialog_entry(char *title, char *labeltxt, char *dialogtxt,
 	gtk_window_set_title(GTK_WINDOW(*toplev),
 			     (title) ? title : "Entry Dialog Title");
 	if (destroy_cb)
-		gtk_signal_connect(GTK_OBJECT(*toplev), "destroy",
-				   GTK_SIGNAL_FUNC(destroy_cb), destroy_cbdata);
+		(void)gtk_signal_connect(GTK_OBJECT(*toplev), "destroy",
+					 GTK_SIGNAL_FUNC(destroy_cb),
+					 destroy_cbdata);
 
 	gtk_container_border_width(GTK_CONTAINER(*toplev), 6);
 	vbox = GTK_DIALOG(*toplev)->vbox;
@@ -246,54 +301,153 @@ get_dialog_entry(char *title, char *labeltxt, char *dialogtxt,
 	gtk_box_pack_start(GTK_BOX(hbox), *entry, FALSE, TRUE, 10);
 
 	hbox = GTK_DIALOG(*toplev)->action_area;
-	tmp = misc_button_new_w_label("Ok", ok_cb, ok_cbdata, hbox,
-#ifdef GTK_HAVE_FEATURES_1_1_0
-				      GTK_RELIEF_NORMAL,
-#endif
-				      NULL, 0, 0, 0, 0);
-
-	tmp = misc_button_new_w_label("Cancel", cancel_cb, cancel_cbdata, hbox,
-#ifdef GTK_HAVE_FEATURES_1_1_0
-				      GTK_RELIEF_NORMAL,
-#endif
-				      NULL, 0, 0, 0, 0);
+	(void)misc_button_new_w_label(_("Ok"), GNOME_STOCK_BUTTON_OK,
+				      GTK_SIGNAL_FUNC(ok_cb), ok_cbdata, hbox,
+				      PACK_START | PACK_EXPAND | PACK_FILL |
+				      CANCEL_DEFAULT | GRAB_DEFAULT, 0);
+	(void)misc_button_new_w_label(_("Cancel"), GNOME_STOCK_BUTTON_CANCEL,
+				      GTK_SIGNAL_FUNC(cancel_cb),
+				      cancel_cbdata, hbox,
+				      PACK_START | PACK_EXPAND | PACK_FILL |
+				      CANCEL_DEFAULT, 0);
 
 	gtk_widget_show_all(*toplev);
 } /* get_dialog_entry */
 
 
-/*** local function definitions ***/
 /*
- * PRIVATE: def_cb
+ * PUBLIC: do_dialog_yes_no_cancel()
  *
- * callback for all button widgets used in do_dialog().  upon invocation,
- * the callback data contains an array of the widget pointers created in
- * do_dialog().  this array is scanned and if a match is found, the id num
- * in the callback data is set, the top-level dialog widget is destroyed,
- * and the dialog widget is set to NULL.  setting the dialog widget to NULL
- * will cause the while loop in do_dialog() to exit.
+ * wrapper for yes-no-cancel dialog box
  */
-static void
-def_cb(GtkWidget *wgt, gpointer cbdata)
+int
+do_dialog_yes_no_cancel(char *title, char *msg)
 {
-	short i;
-	dlg_data_t *data = (dlg_data_t *)cbdata;
+#ifdef USE_GNOME
+	GtkWidget *dlg;
 
-	g_assert(data != NULL);
-	g_assert(data->num > 0);
-	g_assert(data->wgts != NULL);
-	g_assert(data->id == -1);
+	dlg = gnome_message_box_new(msg,
+				    GNOME_MESSAGE_BOX_QUESTION,
+				    GNOME_STOCK_BUTTON_YES,
+				    GNOME_STOCK_BUTTON_NO,
+				    GNOME_STOCK_BUTTON_CANCEL,
+				    NULL);
+	return (1 + gnome_dialog_run_and_close(GNOME_DIALOG(dlg)));
+#else
+	char *buttons[] = { BUTTON_YES, BUTTON_NO, BUTTON_CANCEL };
 
-	for (i = 0; i < data->num; i++) {
-		if (data->wgts[i] == wgt) {
-			data->id = i;
-			break;
-		}
+	return do_dialog(title, msg, 3, buttons, 3, NULL, NULL, TRUE);
+#endif
+} /* do_dialog_yes_no_cancel */
+
+
+/*
+ * PUBLIC: do_dialog_ok()
+ *
+ * wrapper for 'Ok' dialog box
+ */
+int
+do_dialog_ok(char *title, char *msg)
+{
+#ifdef USE_GNOME
+	GtkWidget *dlg;
+
+	dlg = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_INFO,
+				    GNOME_STOCK_BUTTON_OK, NULL);
+
+	return (1 + gnome_dialog_run_and_close(GNOME_DIALOG(dlg)));
+#else
+	char *buttons[] = { _("Ok") };
+
+	return do_dialog(title, msg, 1, buttons, 1, NULL, NULL, TRUE);
+#endif
+} /* do_dialog_ok */
+
+
+void
+do_dialog_clist(GtkWidget **toplev,
+		char *title,
+		int height, int width,
+		GtkWidget **clist, int numcols,
+		char *titles[],
+		void (*destroy_cb)(GtkWidget *, gpointer),
+		gpointer destroy_cbdata,
+		int (*select_cb)(GtkWidget *, unsigned, int,
+				  GdkEventButton *, gpointer),
+		gpointer select_cbdata,
+		void (*close_sel)(GtkWidget *, gpointer),
+		gpointer close_sel_cbdata,
+		void (*close_all)(GtkWidget *, gpointer),
+		gpointer close_all_cbdata)
+{
+	GtkWidget *tmp, *vbox, *hbox, *scrolled_win;
+	int i;
+
+	/* create top level and vbox to hold everything */
+	*toplev = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_title(GTK_WINDOW(*toplev), title);
+	(void)gtk_signal_connect(GTK_OBJECT(*toplev), "destroy",
+				 GTK_SIGNAL_FUNC(destroy_cb), destroy_cbdata);
+	vbox = gtk_vbox_new(FALSE, 10);
+	gtk_container_add(GTK_CONTAINER(*toplev), vbox);
+	gtk_container_border_width(GTK_CONTAINER(vbox), 5);
+
+	/* create clist */
+	*clist = gtk_clist_new_with_titles(numcols, titles);
+	for (i = 0; i < numcols; i++) {
+#ifdef GTK_HAVE_FEATURES_1_1_0
+		gtk_clist_set_column_auto_resize(GTK_CLIST(*clist), i, TRUE);
+#endif
+		gtk_clist_set_column_justification(GTK_CLIST(*clist), i,
+						   GTK_JUSTIFY_LEFT);
 	}
+	gtk_clist_column_titles_passive(GTK_CLIST(*clist));
+	scrolled_win = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scrolled_win), *clist);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
+	gtk_widget_show(scrolled_win);
+	(void)gtk_signal_connect(GTK_OBJECT(*clist), "select_row",
+				 GTK_SIGNAL_FUNC(select_cb), select_cbdata);
 
-	gtk_widget_destroy(data->dlg);
-	data->dlg = NULL;	/* this exits the while loop in do_dialog() */
-} /* def_cb */
+	/* create a separator and then some handy buttons */
+	tmp = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, TRUE, 0);
+	hbox = gtk_hbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+	(void)misc_button_new_w_label(_(" Close Selected "), NULL,
+				      GTK_SIGNAL_FUNC(close_sel),
+				      close_sel_cbdata, hbox, 
+				      PACK_START | PACK_FILL | PACK_EXPAND |
+				      CANCEL_DEFAULT, 2);
+
+	(void)misc_button_new_w_label(_(" Close All "), NULL,
+				      GTK_SIGNAL_FUNC(close_all),
+				      close_all_cbdata, hbox, 
+				      PACK_START | PACK_FILL | PACK_EXPAND |
+				      CANCEL_DEFAULT, 2);
+
+	/* XXX - should not only be fore Window List */
+	(void)misc_button_new_w_label(_(" Exit Window List "), NULL,
+				      GTK_SIGNAL_FUNC(destroy_cb),
+				      destroy_cbdata, hbox, 
+				      PACK_START | CANCEL_DEFAULT |
+				      GRAB_DEFAULT, 2);
+
+	/*
+	 * there should be a way to auto-detect how wide to make these widgets
+	 * instead of having to use arbitrary values
+	 */
+#ifdef GTK_HAVE_FEATURES_1_1_0
+	gtk_window_set_default_size(GTK_WINDOW(*toplev), width, height);
+#else
+	gtk_widget_set_usize(*toplev, width, height);
+#endif
+
+} /* do_dialog_clist */
 
 
 /* the end */
