@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: dialog.h,v 1.6 1999/01/10 21:52:17 kahn Exp kahn $
+ * $Id: dialog.h,v 1.10 2000/01/19 17:19:50 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ extern "C" {
 
 
 #include <gtk/gtk.h>
+#include "main.h"
 
 #define BUTTON_OK	"Ok"
 #define BUTTON_CANCEL	"Cancel"
@@ -44,17 +45,38 @@ typedef struct {
 	GtkWidget *entry;
 } dialog_entry_t;
 
-
 extern int do_dialog_yes_no_cancel(char *title, char *msg);
 extern int do_dialog_ok(char *title, char *msg);
-extern int do_dialog(char *title, char *msg, short num, char **buttons,
-	short dflt, GtkSignalFunc **cbs, void **cbd, bool_t block);
 extern void get_dialog_entry(char *title, char *labeltxt, char *dialogtxt,
 	int entry_len, char *entrytxt,
 	GtkSignalFunc ok_cb, gpointer ok_cbdata,
 	GtkSignalFunc cancel_cb, gpointer cancel_cbdata,
 	GtkSignalFunc destroy_cb, gpointer destroy_cbdata,
 	GtkWidget **toplev, GtkWidget **entry);
+extern int do_dialog(char *title, char *msg, short num, char **buttons,
+	short dflt, GtkSignalFunc **cbs, void **cbd, bool_t block);
+
+extern void do_dialog_clist(
+		GtkWidget **toplev,
+		char *title,
+		int height, int width,
+		GtkWidget **clist, int numcols,
+		char *titles[],
+		void (*destroy_cb)(GtkWidget *, gpointer),
+		gpointer destroy_cbdata,
+		int (*select_cb)(GtkWidget *, unsigned, int,
+				 GdkEventButton *, gpointer),
+		gpointer select_cbdata,
+		void (*close_sel)(GtkWidget *, gpointer),
+		gpointer close_sel_cbdata,
+		void (*close_all)(GtkWidget *, gpointer),
+		gpointer close_all_cbdata);
+
+#ifdef USE_GNOME
+extern int do_dialog_error(char *title, char *msg);
+#else
+#define do_dialog_error	do_dialog_ok
+#endif
 
 #ifdef __cplusplus
 }
