@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: file.c,v 1.20 1999/01/18 19:45:53 kahn Exp kahn $
+ * $Id: file.c,v 1.44 2000/02/26 03:17:33 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <utime.h>
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -32,6 +31,7 @@
 #include <sys/file.h>
 #include <sys/types.h>
 #include <sys/file.h>
+#include "main.h"
 #include "win.h"
 #include "doc.h"
 #include "dialog.h"
@@ -39,6 +39,9 @@
 #include "msgbox.h"
 #include "file.h"
 #include "prefs.h"
+#include "misc.h"
+#include "undo.h"
+#include "gnpintl.h"
 
 #define MAXLOCKATTEMPT 5
 
@@ -47,6 +50,12 @@
 extern void win_set_title(doc_t *d);
 
 
+#ifdef USE_BACKUP
+static void file_backup(char *fname);
+#endif	/* USE_BACKUP */
+static bool_t file_text_xmhtml_read(doc_t *d);
+
+
 /*** global function definitions ***/
 /*
  * PUBLIC: file_exist
@@ -71,12 +80,19 @@ file_exist(char *fname)
  * returns true/false if file is readonly.  requires filename.
  */
 bool_t
-file_is_readonly(char *fname)
+file_is_readonly(doc_t *d)
 {
-	if (strcmp(fname, UNTITLED) == 0 || access(fname, F_OK))
+#ifdef USE_GTKXMHTML
+	if (GTK_IS_XMHTML(d->data))
+		return TRUE;
+#endif
+	if (d->force_ro)
+		return TRUE;
+
+	if (!d->fname || access(d->fname, F_OK))
 		return FALSE;
 
-	return (access(fname, W_OK) != 0);
+	return (access(d->fname, W_OK) != 0);
 } /* file_is_readonly */
 
 
@@ -84,9 +100,7 @@ file_is_readonly(char *fname)
  * PUBLIC: file_full_pathname_make
  *
  * given a filename, tries to construct the full pathname to the file.  if
- * successfuly, returns a newly allocated buffer with the full pathname.
- * if the filename is already a full pathname, the pointer to the filename
- * is returned as is (so no new buffer is allocated).
+ * successful, returns a newly allocated buffer with the full pathname.
  */
 char *
 file_full_pathname_make(char *fname)
@@ -94,6 +108,9 @@ file_full_pathname_make(char *fname)
 	char *full, *cwd;
 	int len;
 
+	if (!fname)
+		return NULL;
+
 	if (fname[0] == '/')
 		return g_strdup(fname);
 
@@ -104,142 +121,212 @@ file_full_pathname_make(char *fname)
 		cwd[strlen(cwd) - 1] = '\0';
 
 	len = strlen(cwd) + strlen(fname) + 2;
-	full = (char *)g_malloc(len);
+	full = g_new(char, len);
 	if ((strlen(fname) > 2) && fname[0] == '.' && fname[1] == '/')
 		g_snprintf(full, len, "%s/%s", cwd, fname + 2);
 	else
 		g_snprintf(full, len, "%s/%s", cwd, fname);
-	g_free(cwd);
+
+	/* use free() here, not g_free(), since getcwd() uses malloc() */
+	free(cwd);
 
 	return full;
 } /* file_full_pathname_make */
 
 
+struct stat *
+file_get_stats(char *fname)
+{
+	struct stat *sb = NULL;
+
+	/* get file stats (e.g., file size is used by files list window) */
+	if (fname) {
+		sb = g_new(struct stat, 1);
+		if (stat(fname, sb) == -1) {
+			g_free(sb);
+			return NULL;
+		}
+	}
+
+	return sb;
+} /* file_get_stats */
+
+
 #ifdef APP_GNP
 /*
  * PUBLIC: file_open_execute
  *
  * actually do the low-level file opening and stuff it into the document's
- * text widget.
+ * text widget.  note that the GTK text widget is rather primitive, so this
+ * routine ends up reading the file text into a buffer first, and then
+ * inserting that buffer using gtk_text_insert(), which basically does another
+ * strcpy().  a better way would have been to mmap() the file, and then read it
+ * directly.  but oh well.
+ *
+ * returns 0 on success, 1 on failure.
  */
 int
-file_open_execute(win_t *w, doc_t *d)
+file_open_execute(doc_t *d)
 {
-	size_t size;
-	off_t bytesread;
-	long num;
-	char *buf;
-	FILE *fp;
-
-	/* get file stats (e.g., file size is used by files list window) */
-	if (strcmp(d->fname, UNTITLED)) {
-		d->sb = (struct stat *)g_malloc(sizeof(struct stat));
-		if (stat(d->fname, d->sb) == -1) {
-			g_free(d->sb);
-			d->sb = NULL;
-		}
-	}
+	if (!d->sb)
+		d->sb = file_get_stats(d->fname);
 
 	if (d->sb && d->sb->st_size > 0) {
-		/* open file */
-		if ((fp = fopen(d->fname, "r")) == NULL) {
-			char msg[255];
-
-			g_snprintf(msg, 255,
-				"file_open_execute: could not open '%s'",
-				d->fname);
-			perror(msg);
-			return 1;
+		if (d->sb && S_ISDIR(d->sb->st_mode)) {
+			msgbar_printf(d->w, _("'%s' is a directory"), d->fname);
+			msgbox_printf(_("file_open_execute: '%s' is a directory"),
+				      d->fname);
+			goto out;
 		}
-		num = file_lock(d->fname, fp, FALSE, FALSE, TRUE);
-		gtk_text_freeze(GTK_TEXT(d->data));
 
-#ifdef USE_SIMPLE_READ_ALGORITHM
-#define READ_CHUNK	32767
-		/*
-		 * the simple read algorithm basically just reads in READ_CHUNK
-		 * bytes at a time.  we can still do better, but this a >>lot<<
-		 * better than before, where we were reading 10 bytes at a
-		 * time!  ideally, we'd like to alloc a buffer to read the
-		 * entire file in at once, or a buffer as large as possible if
-		 * the file is really really large.
-		 */
-		size = READ_CHUNK;
-		buf = (char *)g_malloc(size+1);
+#ifdef USE_GTKXMHTML
+		if (GTK_IS_TEXT(d->data) || GTK_IS_XMHTML(d->data)) {
+			if (!file_text_xmhtml_read(d))
+				return 1;
+		}
 #else
-		/*
-		 * the more aggressive/intelligent method is to try to allocate
-		 * a buffer to fit all the contents of the file; that way, we
-		 * only read the file once.  basically, we try to allocate a
-		 * buffer that is twice the number of bytes that is needed.
-		 * this is because gtk_text_insert() does a memcpy() of the
-		 * read buffer into the text widget's buffer, so we must ensure
-		 * that there is enough memory for both the read and the
-		 * memcpy().  since we'd like to read the entire file at once,
-		 * so we start with a buffer that is twice the filesize.  if
-		 * this fails (e.g., we're reading a really large file), we
-		 * keep reducing the size by half until we are able to get a
-		 * buffer.
-		 */
-		size = 2 * (d->sb->st_size + 1);
-		while ((buf = (char *)malloc(size)) == NULL)
-			size /= 2;
-
-		GNPDBG_FILE(("size %lu is ok (using half)\n", (gulong)size));
-		g_free(buf);
-		size /= 2;
-		if ((buf = (char *)malloc(size + 1)) == NULL) {
-			perror("file_open_execute: unable to malloc read buf");
-			fclose(fp);
-			return 1;
+		if (GTK_IS_TEXT(d->data)) {
+			if (!file_text_xmhtml_read(d))
+				return 1;
 		}
-#endif	/* ifdef USE_CONSERVATIVE_READ_ALGORITHM */
-
-		GNPDBG_FILE(("malloc'd %lu bytes for read buf\n",
-			     (gulong)(size+1)));
-
-		/* buffer allocated, now actually read the file */
-		bytesread = 0;
-		while (bytesread < d->sb->st_size) {
-			num = fread(buf, 1, size, fp);
-			if (num > 0) {
-				gtk_text_insert(
-					GTK_TEXT(d->data),
-					NULL, NULL, NULL,
-#if 0
-					text_font,
-					&text_fg_color, &text_bg_color,
 #endif
-					buf, num);
-				bytesread += num;
-			} else if (ferror(fp)) {
-				/* need to print warning/error message */
-				perror("read error");
-				break;
-			} else /* if (num == 0) */ {
-				GNPDBG_FILE(("hmm... read zero bytes"));
-			}
+#ifdef USE_GTKHTML
+		else if (GTK_IS_HTML(d->data)) {
+			gtk_html_begin(GTK_HTML(d->data), d->fname);
+			gtk_html_parse(GTK_HTML(d->data));
 		}
-		g_free(buf);
-		num = file_unlock(d->fname, fp);
-		fclose(fp);
-		gtk_text_thaw(GTK_TEXT(d->data));
+#endif
 	} /* filesize > 0 */
 
+out:
 	/* misc settings */
-	gtk_label_set(GTK_LABEL(d->tablabel), doc_basefname(d));
-	gtk_text_set_point(GTK_TEXT(d->data), 0);
+	if (GTK_IS_TEXT(d->data))
+		gtk_text_set_point(GTK_TEXT(d->data), 0);
 
 	return 0;
 } /* file_open_execute */
+#endif	/* APP_GNP */
+
+
+/*
+ * returns TRUE on success, FALSE otherwise
+ */
+static bool_t
+file_text_xmhtml_read(doc_t *d)
+{
+	long num;
+	FILE *fp;
+	size_t size;
+	off_t bytesread;
+	char *buf;
+#ifdef USE_GTKXMHTML
+	GString *htmlsrc = NULL;
 #endif
 
+	/* open file */
+	if ((fp = fopen(d->fname, "r")) == NULL) {
+		char msg[255];
+
+		g_snprintf(msg, 255, "file_open_execute: could not open '%s'",
+			   d->fname);
+		perror(msg);
+		return FALSE;
+	}
+	(void)file_lock(d->fname, fp, FALSE, FALSE, TRUE);
+
+
+	if (GTK_IS_TEXT(d->data))
+		gtk_text_freeze(GTK_TEXT(d->data));
+#ifdef USE_GTKXMHTML
+	else if (GTK_IS_XMHTML(d->data))
+		htmlsrc = g_string_new(NULL);
+#endif
+
+#if 0
+	/*
+	 * the more aggressive/intelligent method is to try to allocate a
+	 * buffer to fit all the contents of the file; that way, we only read
+	 * the file once.  basically, we try to allocate a buffer that is twice
+	 * the number of bytes that is needed.  this is because
+	 * gtk_text_insert() does a memcpy() of the read buffer into the text
+	 * widget's buffer, so we must ensure that there is enough memory for
+	 * both the read and the memcpy().  since we'd like to read the entire
+	 * file at once, so we start with a buffer that is twice the filesize.
+	 * if this fails (e.g., we're reading a really large file), we keep
+	 * reducing the size by half until we are able to get a buffer.
+	 *
+	 * Note: this causes memory fragmentation on some systems, so just use
+	 * the simplistic method, which isn't all that bad.
+	 */
+	size = 2 * (d->sb->st_size + 1);
+	while ((buf = g_new(char, size)) == NULL)
+		size /= 2;
+
+	GNPDBG_FILE(("size %lu is ok (using half)\n", (gulong)size));
+	g_free(buf);
+	size /= 2;
+	if ((buf = g_new(char, size + 1)) == NULL) {
+		perror("file_open_execute: unable to malloc read buf");
+		fclose(fp);
+		return FALSE;
+	}
+	GNPDBG_FILE(("malloc'd %lu bytes for read buf\n", (gulong)(size+1)));
+#endif
+	size = (size_t)prefs.read_size;
+	buf = g_new(char, size + 1);
+
+	/* buffer allocated, now actually read the file */
+	bytesread = 0;
+	while (bytesread < d->sb->st_size) {
+		/* flush any pending events */
+		gtk_main_iteration_do(FALSE);
+
+		num = fread(buf, 1, size, fp);
+		if (num > 0) {
+			if (GTK_IS_TEXT(d->data))
+				gtk_text_insert(GTK_TEXT(d->data), NULL, NULL,
+						NULL, buf, num);
+#ifdef USE_GTKXMHTML
+			else if (GTK_IS_XMHTML(d->data))
+				htmlsrc = g_string_append(htmlsrc, buf);
+#endif
+			bytesread += num;
+		} else if (ferror(fp)) {
+			perror("read error");
+			(void)do_dialog_error(
+					_("Read error!"),
+					_(" file_open_execute: read error! "));
+			break;
+		} else if (feof(fp)) {
+			break;
+		} else /* if (num == 0) */ {
+			g_warning("file_open_execute: read 0 bytes\n");
+			break;
+		}
+	} /* while more bytes to read */
+	g_free(buf);
+
+	(void)file_unlock(d->fname, fp);
+	(void)fclose(fp);
+	if (GTK_IS_TEXT(d->data))
+		gtk_text_thaw(GTK_TEXT(d->data));
+#ifdef USE_GTKXMHTML
+	else if (GTK_IS_XMHTML(d->data)) {
+		g_assert(htmlsrc);
+		gtk_xmhtml_source(GTK_XMHTML(d->data), htmlsrc->str);
+		g_string_free(htmlsrc, TRUE);
+	}
+#endif
+
+	return TRUE;
+} /* file_text_xmhtml_read */
+
 
 /*
  * PUBLIC: file_perm_string
  *
- * returns the rwx permissions of a file in a string.  non-reentrant.
- * does not look at setgid, setuid, or sticky bit settings.
+ * returns the rwx permissions of a file in a string.  non-reentrant.  does not
+ * look at setgid, setuid, or sticky bit settings.
  */
 char *
 file_perm_string(struct stat *sb)
@@ -272,21 +359,71 @@ bool_t
 file_save_execute(doc_t *d, bool_t saveas)
 {
 	FILE *fp;
-	int ret;
 	char *buf;
+	long oldperm = -1;
+	uid_t  olduid = (uid_t)-1;
+	gid_t  oldgid = (gid_t)-1;
+#if defined(USE_UNDOREDO) && defined(GTK_HAVE_FEATURES_1_1_0)
+	GList *lp;
+#endif
 
-	if (!saveas && file_is_readonly(d->fname)) {
-		(void)do_dialog_ok("Read only file!",
-			" Cannot save file.  File is read only! ");
+	g_assert(d->fname);
+	if (d->sb == NULL)
+		d->sb = g_new(struct stat, 1);
+	else {
+		oldperm = d->sb->st_mode;
+		olduid = d->sb->st_uid;
+		oldgid = d->sb->st_gid;
+	}
+	if (stat(d->fname, d->sb) == -1) {
+		g_free(d->sb);
+		d->sb = NULL;
+	}
+
+	if (d->sb && S_ISDIR(d->sb->st_mode)) {
+		(void)do_dialog_error( _("File is a directory!"),
+				      _(" Cannot save.  File is a directory! "));
 		return FALSE;
 	}
 
+	if (!saveas && file_is_readonly(d)) {
+		(void)do_dialog_error( _("Read only file!"),
+				      _(" Cannot save file. File is read only! "));
+		return FALSE;
+	}
+
+#ifdef USE_GTKXMHTML
+	if (GTK_IS_XMHTML(d->data)) {
+		(void)do_dialog_error( _("Can't save HTML!"),
+				      _(" Can't save HTML files! "));
+		return TRUE;
+	}
+#endif
+
+	/* if the file has no changes, then there's no need to save anything */
+	if (!d->changed) {
+		msgbar_printf(d->w, _("%s: no changes to save"), doc_basefname(d));
+		msgbox_printf(_("%s: no changes to save"), doc_basefname(d));
+		return TRUE;
+	}
+
+#ifdef USE_BACKUP
+	if (IS_DO_BACKUP())
+		file_backup(d->fname);
+#endif	/* USE_BACKUP */
+
 	if ((fp = fopen(d->fname, "w")) == NULL) {
 		g_warning("file_save_execute: can't open '%s' for saving",
-			d->fname);
+			  d->fname);
 		return FALSE;
 	}
-	ret = file_lock(d->fname, fp, TRUE, FALSE, TRUE);
+	(void)file_lock(d->fname, fp, TRUE, FALSE, TRUE);
+
+	/* try to set permissions to original. maybe use fchmod() and fchown? */
+	if (oldperm >= 0)
+		(void)chmod(d->fname, (mode_t)oldperm);
+	(void)chown((char *)d->fname, olduid, oldgid);
+
 	gtk_text_thaw(GTK_TEXT(d->data));
 
 	/*
@@ -305,31 +442,41 @@ file_save_execute(doc_t *d, bool_t saveas)
 		return FALSE;
 	}
 	g_free(buf);
-	fflush(fp);
-	ret = file_unlock(d->fname, fp);
-	fclose(fp);
-
-	if (d->sb == NULL)
-		d->sb = (struct stat *)g_malloc(sizeof(struct stat));
-	if (stat(d->fname, d->sb) == -1) {
-		g_free(d->sb);
-		d->sb = NULL;
-	}
+	(void)fflush(fp);
+	(void)file_unlock(d->fname, fp);
+	(void)fclose(fp);
 
 	gtk_label_set(GTK_LABEL(d->tablabel), doc_basefname(d));
 	d->changed = FALSE;
-	if (!d->changed_id)
-		d->changed_id = gtk_signal_connect(
-					GTK_OBJECT(d->data), "changed",
-					GTK_SIGNAL_FUNC(doc_changed_cb), d);
+	if (d->changed_id)
+		gtk_signal_disconnect(GTK_OBJECT(d->data),  d->changed_id);
+	d->changed_id = gtk_signal_connect(
+				GTK_OBJECT(d->data), "changed",
+				GTK_SIGNAL_FUNC(doc_changed_cb), d);
+
+#if defined(USE_UNDOREDO) && defined(GTK_HAVE_FEATURES_1_1_0)
+	/*
+	 * update the undo and redo lists.  since we have successfully saved
+	 * the file at this point, any further undo's or redo's always results
+	 * in the document being "changed".
+	 */
+	for (lp = d->undolist; lp; lp = g_list_next(lp)) {
+		undo_t *undo = (undo_t *)lp->data;
+		undo->changed = TRUE;
+	}
+	for (lp = d->redolist; lp; lp = g_list_next(lp)) {
+		undo_t *undo = (undo_t *)lp->data;
+		undo->changed = TRUE;
+	}
+#endif	/* USE_UNDOREDO */
 
 	win_set_title(d);
 
-	msgbar_printf(d->w, "Saved %s", doc_basefname(d));
-	msgbox_printf("saved %s", doc_basefname(d));
+	msgbar_printf(d->w, _("Saved %s"), doc_basefname(d));
+	msgbox_printf(_("saved %s"), doc_basefname(d));
 	return TRUE;
 } /* file_save_execute */
-#endif
+#endif	/* APP_GNP */
 
 
 
@@ -349,8 +496,8 @@ int
 file_lock(const char *path, FILE *fp, bool_t excl, bool_t dot, bool_t timeout)
 {
 	int r = 0;
-	int fd = fileno(fp);
 #if defined (USE_FCNTL) || defined (USE_FLOCK)
+	int fd = fileno(fp);
 	int count;
 	int attempt;
 	struct stat prev_sb;
@@ -453,13 +600,15 @@ file_lock(const char *path, FILE *fp, bool_t excl, bool_t dot, bool_t timeout)
  * PUBLIC: file_unlock
  *
  * this routine was taken from mutt (www.mutt.org).  uses either fcntl() or
- * flock() to unlock a file previously locked by lock_file().
- * dot-locking code is currently not supported.
+ * flock() to unlock a file previously locked by lock_file().  dot-locking code
+ * is currently not supported.
  */
 int 
 file_unlock(const char *path, FILE *fp)
 {
+#if defined (USE_FCNTL) || defined (USE_FLOCK)
 	int fd = fileno(fp);
+#endif
 #ifdef USE_FCNTL
 	struct flock unlockit = {F_UNLCK, 0, 0, 0};
 
@@ -481,9 +630,75 @@ file_unlock(const char *path, FILE *fp)
 } /* file_unlock */
 
 
+/*** local function defintions ***/
+#ifdef USE_BACKUP
+/*
+ * PRIVATE: file_backup
+ *
+ * creates a backup file.  first tries to remove the old backup file, if one
+ * exists.  if successful or if one never existed, renames the current file to
+ * the backup name.
+ */
+static void
+file_backup(char *fname)
+{
+	char *backupname;
+	char *backupdir;
+	char *dname = NULL;
+	int len;
+
+	if (fname == NULL || !file_exist(fname))
+		return;
+
+	GNPDBG_FILE(("file_backup: orig file = '%s'\n", fname));
+	if (prefs.backupdir && prefs.backupdir != "" &&
+	    strcmp(prefs.backupdir, "(null)")) {
+		backupdir = prefs.backupdir;
+	} else {
+		if (fname[0] == '/')
+			dname = my_dirname((const gchar *)fname);
+		else {
+			char *tmp = file_full_pathname_make(fname);
+			dname = my_dirname((const gchar *)tmp);
+			g_free(tmp);
+		}
+		backupdir = dname;
+	}
+	len = 2 + strlen(backupdir) + strlen(my_basename(fname)) +
+	      (prefs.backupsuffix ? strlen(prefs.backupsuffix) : 1);
+	backupname = g_new(char, len);
+
+	g_snprintf(backupname, len, "%s/%s%s",
+		   backupdir, my_basename(fname),
+		   (prefs.backupsuffix) ? prefs.backupsuffix : NULL);
+	if (dname)
+		free(dname);	/* don't use g_free() */
+	GNPDBG_FILE(("file_backup: backup file = '%s'\n", backupname));
+
+	if (file_exist(backupname)) {
+		if (unlink(backupname) == -1) {
+			perror("file_backup: unlink() error");
+			(void)do_dialog_error("unlink() failed!",
+					      " Could not remove old backup! ");
+			g_free(backupname);
+			return;
+		}
+	}
+	
+	if (file_exist(fname) && rename(fname, backupname) == -1) {
+		perror("file_backup: rename() error");
+		(void)do_dialog_error("rename() failed!",
+				      " Could not create backup! ");
+	}
+
+	g_free(backupname);
+} /* file_backup */
+#endif	/* USE_BACKUP */
+
+
 /****** dotlocking is currently not supported.  eventually, it probably will
 	be, which is why the code is still here *****/
-#ifdef USE_DOTLOCK
+#if defined(USE_DOTLOCK) && defined(FALSE)
 /* HP-UX and ConvexOS don't have this macro */
 #ifndef S_ISLNK
 #define S_ISLNK(x) (((x) & S_IFMT) == S_IFLNK ? 1 : 0)
@@ -540,7 +755,7 @@ dotlock_file(const char *path, int retry)
 
 	while ((fd = open(nfslockfile, O_WRONLY | O_EXCL | O_CREAT, 0)) < 0) {
 		if (errno != EAGAIN) {
-			mutt_perror("cannot open NFS lock file!");
+			mutt_perror( _("cannot open NFS lock file!"));
 			return (-1);
 		}
 	}
@@ -566,7 +781,7 @@ dotlock_file(const char *path, int retry)
 		/* only try to remove the lock if the file is not changing */
 		if (prev_size == sb.st_size && ++count >=
 			(retry ? MAXLOCKATTEMPT : 0)) {
-			if (retry && mutt_yesorno("Lock count exceeded, remove lock?", 1) == 1) {
+			if (retry && mutt_yesorno( _("Lock count exceeded, remove lock?"), 1) == 1) {
 				unlink(lockfile);
 				count = 0;
 				attempt = 0;
@@ -576,7 +791,7 @@ dotlock_file(const char *path, int retry)
 		}
 		prev_size = sb.st_size;
 
-		printf("Waiting for lock attempt #%d...", ++attempt);
+		printf( _("Waiting for lock attempt #%d..."), ++attempt);
 		sleep(1);
 	}
 
