@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: file.h,v 1.9 1998/10/30 22:03:42 kahn Exp kahn $
+ * $Id: file.h,v 1.12 1999/09/12 14:20:57 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -31,13 +31,14 @@ extern "C" {
 #include "doc.h"
 
 extern bool_t file_exist(char *fname);
-extern int file_open_execute(win_t *w, doc_t *d);
+extern int file_open_execute(doc_t *d);
 extern char *file_perm_string(struct stat *sb);
 extern bool_t file_save_execute(doc_t *d, bool_t saveas);
 extern char * file_full_pathname_make(char *fname);
 extern int file_unlock(const char *path, FILE *fp);
 extern int file_lock(const char *, FILE *fp, bool_t, bool_t, bool_t);
-extern bool_t file_is_readonly(char *fname);
+extern bool_t file_is_readonly(doc_t *d);
+extern struct stat *file_get_stats(char *fname);
 
 
 #ifdef __cplusplus
