@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: menu.h,v 1.4 1999/01/18 20:02:31 kahn Exp kahn $
+ * $Id: menu.h,v 1.11 2000/02/28 17:22:54 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -29,20 +29,18 @@ extern "C" {
 #include <glib.h>
 #include "win.h"
 
-typedef struct {
-	char *name;
-	GtkMenuCallback cb;
-} qmenu_t;
-
-
-extern void menu_main_init(win_t *w);
-#ifdef APP_GNP
-extern void quickmenu_init(win_t *w);
+extern void	menu_main_init(win_t *w);
+extern void	quickmenu_init(win_t *w);
+extern gboolean	quickmenu_show(GtkWidget *wgt, GdkEvent *ev);
+#ifdef USE_RECENT
+extern void	menu_recent_doc_add(GtkMenuCallback cbf, gpointer cbd);
+extern void	menu_recent_doc_remove(gpointer dnnp);
+#endif	/* USE_RECENT */
+#ifdef GTK_HAVE_FEATURES_1_1_0
+extern void	menu_main_state_set(void *data);
+#else
+# define	menu_main_state_set(data)
 #endif
-extern gboolean quickmenu_show(GtkWidget *wgt, GdkEvent *ev);
-extern void menu_recent_doc_add(GtkMenuCallback cbf, gpointer cbd);
-extern void menu_recent_doc_remove(gpointer dnnp);
-
 
 #ifdef __cplusplus
 }
