@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: msgbar.c,v 1.11 1999/01/06 20:55:24 kahn Exp kahn $
+ * $Id: msgbar.c,v 1.17 1999/11/09 16:32:13 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,14 @@
  */
 #include <stdarg.h>
 #include <string.h>
+#include "main.h"
+#ifdef USE_GNOME
+#include <gnome.h>
+#endif
 #include "win.h"
 #include "msgbar.h"
 #include "prefs.h"
+#include "menu.h"
 
 
 /*** global function definitions ***/
@@ -41,6 +46,7 @@ msgbar_clear(gpointer data)
 } /* msgbar_clear */
 
 
+#ifndef USE_GNOME
 /*
  * PUBLIC: msgbar_init
  *
@@ -57,6 +63,7 @@ msgbar_init(win_t *w, GtkWidget *parent)
 	if (IS_SHOW_MSGBAR())
 		gtk_widget_show(w->msgbar);
 } /* msgbar_init */
+#endif
 
 
 /*
@@ -87,13 +94,17 @@ msgbar_printf(win_t *w, const char *fmt, ...)
 	va_end(ap2);
 #endif
 	if (w->lastmsg == NULL || (w->lastmsg && strcmp(w->lastmsg, msg))) {
-		if (w->lastmsg)
+		if (w->lastmsg && w->timeout_id > 0)
 			gtk_timeout_remove(w->timeout_id);
+#ifdef USE_GNOME
+		gnome_appbar_set_status(GNOME_APPBAR(w->msgbar), msg);
+#else
 		gtk_label_set(GTK_LABEL(w->msgbar), msg);
-		if (w->lastmsg)
-			g_free(w->lastmsg);
+#endif
+		g_free(w->lastmsg);
 		w->lastmsg = g_strdup(msg);
-		msgbar_timeout_add(w);
+		w->timeout_id = gtk_timeout_add(
+					5000, (GtkFunction)msgbar_clear, w);
 	}
 #ifdef GTK_HAVE_FEATURES_1_1_0
 	g_free(msg);
@@ -112,12 +123,21 @@ msgbar_redraw(void *data)
 	win_t *w = (win_t *)data;
 
 	if (IS_SHOW_MSGBAR()) {
-		gtk_widget_show(w->msgbar);
-		gtk_widget_show(w->docinfo);
+#ifdef USE_GNOME
+		gtk_widget_show_all(w->msgbar->parent);
+#else
+		gtk_widget_show_all(w->hbox_bot);
+#endif
 	} else {
-		gtk_widget_hide(w->msgbar);
-		gtk_widget_hide(w->docinfo);
+#ifdef USE_GNOME
+		gtk_widget_hide_all(w->msgbar->parent);
+#else
+		gtk_widget_hide_all(w->hbox_bot);
+#endif
 	}
+
+	/* need to update the toggle items in the menu */
+	menu_main_state_set(w);
 } /* msgbar_redraw */
 
 
@@ -129,29 +149,12 @@ msgbar_redraw(void *data)
 void
 msgbar_toggle(GtkWidget *wgt, gpointer cbdata)
 {
-	win_t *w = (win_t *)cbdata;
-
-	g_assert(w != NULL);
-	if (IS_SHOW_MSGBAR()) {
-		gtk_widget_hide_all(w->hbox_bot);
+	if (IS_SHOW_MSGBAR())
 		CLEAR_SHOW_MSGBAR();
-	} else {
-		gtk_widget_show_all(w->hbox_bot);
+	else
 		SET_SHOW_MSGBAR();
-	}
+	win_foreach(msgbar_redraw);
 } /* msgbar_toggle */
 
 
-/*
- * PUBLIC: msgbar_timeout_add
- *
- * after ~3 seconds, clears the message bar.
- */
-void
-msgbar_timeout_add(win_t *w)
-{
-	w->timeout_id = gtk_timeout_add(3000, (GtkFunction)msgbar_clear, w);
-} /* msgbar_timeout_add */
-
-
 /* the end */
