@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: msgbox.c,v 1.16 1999/01/11 16:33:39 kahn Exp kahn $
+ * $Id: msgbox.c,v 1.27 2000/02/29 03:26:19 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,17 +17,20 @@
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include "main.h"
+#ifdef WANT_MSGBOX
 #include <stdio.h>
 #include <stdarg.h>
 #include <time.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib.h>
-#include "main.h"
 #include "prefs.h"
 #include "dialog.h"
+#include "misc.h"
 #include "msgbox.h"
 
+#include "gnpintl.h"
 
 /*** local types ***/
 #define NUM_DEL_MSGS	(prefs.msgbox_max_msg * (prefs.msgbox_per_del / 100.0))
@@ -46,7 +49,6 @@ static msgbox_t msgbox = { NULL, NULL, 0 };
 /*** local function prototypes ***/
 static void msgbox_append(char *msg);
 static void msgbox_clear(void);
-static void msgbox_destroy(GtkWidget *w, gpointer data);
 static void msgbox_truncate(void);
 #ifdef GNP_DBG_MSGBOX
 static void msgbox_append_dbg(GtkWidget *w, gpointer data);
@@ -80,14 +82,7 @@ msgbox_close(void)
 void
 msgbox_init(void)
 {
-	GtkWidget *box1;
-	GtkWidget *box2;
-	GtkWidget *hbox;
-	GtkWidget *button;
-	GtkWidget *separator;
-	GtkWidget *table;
-	GtkWidget *vscrollbar;
-	GtkWidget *text;
+	GtkWidget *box1, *box2, *hbox, *tmp, *table;
 
 	if (!IS_USE_MSGBOX() || msgbox.toplev != NULL)
 		return;
@@ -98,11 +93,11 @@ msgbox_init(void)
 	gtk_widget_set_name(msgbox.toplev, "messages");
 	gtk_widget_set_usize(msgbox.toplev, 500, 250);
 	gtk_window_set_policy(GTK_WINDOW(msgbox.toplev), TRUE, TRUE, FALSE);
-	gtk_window_set_title(GTK_WINDOW(msgbox.toplev), "Messages");
+	gtk_window_set_title(GTK_WINDOW(msgbox.toplev), _("Messages"));
 	gtk_container_border_width(GTK_CONTAINER(msgbox.toplev), 0);
 
-	gtk_signal_connect(GTK_OBJECT(msgbox.toplev), "destroy",
-			   GTK_SIGNAL_FUNC(msgbox_destroy), msgbox.toplev);
+	(void)gtk_signal_connect(GTK_OBJECT(msgbox.toplev), "delete_event",
+				 GTK_SIGNAL_FUNC(gtk_true), NULL);
 
 	box1 = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(msgbox.toplev), box1);
@@ -121,8 +116,7 @@ msgbox_init(void)
 	gtk_widget_show(table);
 
 	msgbox.text = gtk_text_new(NULL, NULL);
-	text = msgbox.text;
-	gtk_table_attach(GTK_TABLE(table), text, 0, 1, 0, 1,
+	gtk_table_attach(GTK_TABLE(table), msgbox.text, 0, 1, 0, 1,
 #ifdef GTK_HAVE_FEATURES_1_1_0
 			 (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
 			 (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
@@ -131,10 +125,10 @@ msgbox_init(void)
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 #endif
 			 0, 0);
-	gtk_widget_show(text);
+	gtk_widget_show(msgbox.text);
 
-	vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
-	gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1,
+	tmp = gtk_vscrollbar_new(GTK_TEXT(msgbox.text)->vadj);
+	gtk_table_attach(GTK_TABLE(table), tmp, 1, 2, 0, 1,
 #ifdef GTK_HAVE_FEATURES_1_1_0
 			 (GtkAttachOptions)GTK_FILL,
 			 (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),
@@ -142,15 +136,15 @@ msgbox_init(void)
 			 GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 #endif
 			 0, 0);
-	gtk_widget_show(vscrollbar);
+	gtk_widget_show(tmp);
 
-	gtk_widget_realize(text);
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(text), FALSE);
+	gtk_widget_realize(msgbox.text);
+	gtk_text_set_editable(GTK_TEXT(msgbox.text), FALSE);
+	gtk_text_set_word_wrap(GTK_TEXT(msgbox.text), FALSE);
 
-	separator = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(box1), separator, FALSE, TRUE, 0);
-	gtk_widget_show(separator);
+	tmp = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(box1), tmp, FALSE, TRUE, 0);
+	gtk_widget_show(tmp);
 
 
 	box2 = gtk_vbox_new(FALSE, 10);
@@ -159,33 +153,27 @@ msgbox_init(void)
 	gtk_widget_show(box2);
 
 
-	hbox = gtk_hbutton_box_new();
+	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(box2), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 #ifdef GNP_DBG_MSGBOX
-	button = gtk_button_new_with_label("Add message!");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(msgbox_append_dbg), msgdbg_str);
-	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-	gtk_widget_show(button);
+	(void)misc_button_new_w_label(_("Add message!"), NULL,
+				      msgbox_append_dbg, msgdbg_str, hbox,
+				      PACK_END | PACK_FILL | SHOW_BUTTON |
+				      CANCEL_DEFAULT, 0);
 #endif
 
-	button = gtk_button_new_with_label("Clear");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(msgbox_clear), NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-	gtk_widget_show(button);
+	(void)misc_button_new_w_label(_("Close"), GNOME_STOCK_BUTTON_CLOSE,
+				      msgbox_close, NULL, hbox,
+				      PACK_END | PACK_FILL | CANCEL_DEFAULT |
+				      GRAB_DEFAULT | SHOW_BUTTON, 2);
 
-	button = gtk_button_new_with_label("Close");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(msgbox_close), NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-	gtk_widget_grab_default(button);
-	gtk_widget_show(button);
-
-	msgbox_printf("%s%s %s", WELCOME_MSG, APP_NAME, APP_VERSION);
+	(void)misc_button_new_w_label(_("Clear"), GNOME_STOCK_PIXMAP_CLEAR,
+				      msgbox_clear, NULL, hbox,
+				      PACK_END | PACK_FILL | SHOW_BUTTON |
+				      CANCEL_DEFAULT, 2);
 
+	msgbox_printf("%s%s %s", WELCOME_MSG, APP_NAME, APP_VERSION, _("."));
 } /* msgbox_init */
 
 
@@ -240,19 +228,15 @@ msgbox_show(GtkWidget *w, gpointer data)
 {
 	if (!IS_USE_MSGBOX()) {
 		(void)do_dialog_ok(
-			"No messages!",
-			" Message box has been disabled ");
+			N_("No messages!"),
+			N_(" Message box has been disabled "));
 		return;
 	}
 
-	if (msgbox.toplev == NULL)
-		msgbox_init();
-
-	if (msgbox.toplev == NULL)
+	if (misc_show_and_raise(msgbox.toplev))
 		return;
 
-	gtk_widget_show(msgbox.toplev);
-	gdk_window_raise(msgbox.toplev->window);
+	msgbox_init();
 } /* msgbox_show */
 
 
@@ -285,7 +269,7 @@ msgbox_append(char *msg)
 		timestr[strlen(timestr) - 1] = '\0';
 
 	/* build the time/data as a string, append \n to msg if user didn't */
-	buf = (char *)g_malloc(strlen(timestr) + strlen(msg) + 5);
+	buf = g_new(char, strlen(timestr) + strlen(msg) + 5);
 	g_snprintf(buf, strlen(timestr) + strlen(msg) + 5,
 		   "[%s] %s%c", timestr, msg,
 		   (msg[strlen(msg) - 1] == '\n') ? '\0' : '\n');
@@ -320,24 +304,6 @@ msgbox_clear(void)
 } /* msgbox_clear */
 
 
-/*
- * PRIVATE: msgbox_destroy
- *
- * zap!
- */
-static void
-msgbox_destroy(GtkWidget *w, gpointer data)
-{
-	if (msgbox.toplev == NULL || !IS_USE_MSGBOX())
-		return;
-
-	gtk_widget_destroy(msgbox.text);
-	gtk_widget_destroy(msgbox.toplev);
-	msgbox.text = NULL;
-	msgbox.toplev = NULL;
-} /* msgbox_destroy */
-
-
 /*
  * PRIVATE: msgbox_truncate
  *
@@ -393,4 +359,15 @@ msgbox_append_dbg(GtkWidget *w, gpointer data)
 } /* msgbox_append_dbg */
 #endif
 
+#else
+
+void msgbox_printf(const char *fmt, ...);
+
+void
+msgbox_printf(const char *fmt, ...)
+{
+}
+
+#endif  /* #ifdef WANT_MSGBOX */
+
 /* the end */
