@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: print.c,v 1.15 1999/01/18 20:04:54 kahn Exp kahn $
+ * $Id: print.c,v 1.22 2000/02/19 08:33:28 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "prefs.h"
 #include "misc.h"
 
+#include "gnpintl.h"
 
 /*** local function prototypes ***/
 static char *get_filename(win_t *w);
@@ -58,20 +59,18 @@ print_cb(GtkWidget *widget, gpointer cbdata)
 	char *title;
 
 	g_assert(w != NULL);
-	d = doc_current(w);
+	d = DOC_CURRENT(w);
 	g_assert(d != NULL);
 
-	if (d->printwin) {
-		gdk_window_raise(d->printwin->window);
+	if (misc_show_and_raise(d->printwin))
 		return;
-	}
 
-	title = (char *)g_malloc(strlen(doc_basefname(d)) + 11);
-	sprintf(title, "Print '%s' ?", doc_basefname(d));
+	title = g_new(char, strlen(doc_basefname(d)) + 11);
+	sprintf(title, _("Print '%s' ?"), doc_basefname(d));
 
 	get_dialog_entry(title,
-			 "Enter print command.\nUse '%s' for file name",
-			 "Print Command:",
+			 _("Enter print command.\nUse '%s' for file name"),
+			 _("Print Command:"),
 			 128, prefs.printcmd,
 			 GTK_SIGNAL_FUNC(print_execute), (gpointer)w,
 			 GTK_SIGNAL_FUNC(print_cancel), (gpointer)d,
@@ -103,16 +102,16 @@ get_filename(win_t *w)
 	doc_t *d;
 	char *fname = NULL;
 	char *buttons[] =
-		{ "Print anyway", " Save, then print ", BUTTON_CANCEL };
+		{ _("Print anyway"), _(" Save, then print "), _(BUTTON_CANCEL) };
 	char *title, *msg;
 
 	g_assert(w != NULL);
-	d = doc_current(w);
+	d = DOC_CURRENT(w);
 	g_assert(d != NULL);
 
 	if (!d->changed && d->fname) {
 		if (d->sb == NULL)
-			d->sb = (struct stat *)g_malloc(sizeof(struct stat));
+			d->sb = g_new(struct stat, 1);
 		if (stat(d->fname, d->sb) == -1) {
 			/* print warning */
 			g_free(d->sb);
@@ -125,11 +124,11 @@ get_filename(win_t *w)
 
 		title = g_strdup(PRINT_TITLE);
 		if (d->fname)
-			msg = (char *)g_malloc(strlen(PRINT_MSG) +
-						strlen(d->fname) + 6);
+			msg = g_new(char, strlen(PRINT_MSG) +
+					  strlen(d->fname) + 6);
 		else
-			msg = (char *)g_malloc(strlen(PRINT_MSG) +
-						strlen(UNTITLED) + 6);
+			msg = g_new(char, strlen(PRINT_MSG) +
+					  strlen(UNTITLED) + 6);
 		sprintf(msg, " '%s' %s ", 
 			(d->fname) ? d->fname : UNTITLED, PRINT_MSG);
 
@@ -143,11 +142,10 @@ get_filename(win_t *w)
 			fname = generate_temp_file(d);
 			break;
 		case 2 :
-			if (d->fname == NULL)
+			if (!d->fname)
 				(void)doc_saveas_common(w);
 			else
 				(void)doc_save_common(w);
-
 			fname = g_strdup(d->fname);
 			break;
 		case 3 :
@@ -179,7 +177,7 @@ generate_temp_file(doc_t *d)
 
 	fname = my_tempname();
 
-	oldmask = umask(0600);
+	oldmask = umask(0066);
 	if ((fp = fopen(fname, "w")) == NULL) {
 		g_error("generate_temp_file: unable to open '%s'\n", fname);
 		g_free(fname);
@@ -250,7 +248,7 @@ print_execute(GtkWidget *wgt, gpointer cbdata)
 	win_t *w = (win_t *)cbdata;
 
 	g_assert(w != NULL);
-	d = doc_current(w);
+	d = DOC_CURRENT(w);
 	g_assert(d != NULL);
 
 	/* print using specified command */
@@ -264,12 +262,12 @@ print_execute(GtkWidget *wgt, gpointer cbdata)
 	tmp += 2;
 
 	if ((fname = get_filename(w)) == NULL) {
-		print_destroy(NULL, w->curdoc);
+		gtk_widget_destroy(d->printwin);
 		return;	/* canceled */
 	}
 
-	/* build command and execute; g_malloc handles memory alloc errors */
-	scmd = (char *)g_malloc(strlen(pcmd) + strlen(fname) + 1);
+	/* build command and execute */
+	scmd = g_new(char, strlen(pcmd) + strlen(fname) + 1);
 	sprintf(scmd, "%s%s%s", pcmd, fname, tmp);
 	GNPDBG_PREFS(("print_execute: print command = '%s'\n", scmd));
 	if (system(scmd) == -1)
@@ -281,8 +279,8 @@ print_execute(GtkWidget *wgt, gpointer cbdata)
 			perror("print_execute: unlink() error");
 	g_free(fname);
 
-	msgbar_printf(w, "Printed %s", d->fname);
-	msgbox_printf("printed %s", d->fname);
+	msgbar_printf(w, _("Printed %s"), (d->fname) ? d->fname : UNTITLED);
+	msgbox_printf(_("printed %s"), (d->fname) ? d->fname : UNTITLED);
 
 	gtk_widget_destroy(d->printwin);
 } /* print_execute */
