@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: prjbar.c,v 1.7 1998/11/15 17:46:43 kahn Exp kahn $
+ * $Id: prjbar.c,v 1.14 2000/01/19 17:36:21 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,17 +19,21 @@
  */
 #include <stdarg.h>
 #include <string.h>
+#include "config.h"
 #include "win.h"
+#include "doc.h"
 #include "project.h"
 #include "prjbar.h"
 #include "prefs.h"
+#include "gnpintl.h"
+
 
 #ifdef WANT_PROJECT
 
-#define NO_PROJECT	"Project: NONE / Files: NONE"
-
+static const char *no_project = N_("Project: NONE / Files: NONE");
 
 /*** global function definitions ***/
+#if 0
 /*
  * PUBLIC: prjbar_clear
  *
@@ -43,8 +47,10 @@ prjbar_clear(gpointer data)
 
 	prjbar_update(w);
 } /* prjbar_clear */
+#endif
 
 
+/*** global function definitions ***/
 /*
  * PUBLIC: prjbar_init
  *
@@ -58,7 +64,7 @@ prjbar_init(win_t *w)
 	tmp = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(w->mainbox), tmp, FALSE, TRUE, 0);
 	gtk_widget_show(tmp);
-	w->prjbar = gtk_label_new(NO_PROJECT);
+	w->prjbar = gtk_label_new( _(no_project));
 	gtk_box_pack_start(GTK_BOX(w->mainbox), w->prjbar, FALSE, TRUE, 0);
 	tmp = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(w->mainbox), tmp, FALSE, TRUE, 0);
@@ -79,17 +85,18 @@ void
 prjbar_update(win_t *w)
 {
 	char *msg;
-	prj_t *prj = (prj_t *)(w->prj);
+	doc_t *d = DOC_CURRENT(w);
+	prj_t *prj = d->prj;
 
 	if (prj) {
-		msg = (char *)g_malloc(strlen(prj->name) + 128);
+		msg = g_new(char, strlen(prj->name) + 128);
 		g_snprintf(msg, strlen(prj->name) + 128,
-				"Project: %s / Files: %d",
+				_("Project: %s / Files: %d"),
 				prj->name, prj->numfiles);
 		gtk_label_set(GTK_LABEL(w->prjbar), msg);
 		g_free(msg);
 	} else
-		gtk_label_set(GTK_LABEL(w->prjbar), NO_PROJECT);
+		gtk_label_set(GTK_LABEL(w->prjbar), _(no_project));
 } /* prjbar_update */
 
 
