@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: recent.c,v 1.13 1999/01/11 16:35:38 kahn Exp kahn $
+ * $Id: recent.c,v 1.23 2000/02/25 18:20:37 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include "main.h"
+#ifdef USE_RECENT
 #include <stdio.h>
 #include <string.h>
 #include "doc.h"
@@ -48,9 +50,9 @@ recent_list_add(win_t *w, char *fname)
 	GSList *rlp;
 	doc_t *d;
 
-	d = doc_current(w);
+	d = DOC_CURRENT(w);
 	g_assert(d != NULL);
-	if (strcmp(d->fname, UNTITLED) == 0)
+	if (!d->fname)
 		return;	/* don't add 'Untitled' */
 
 	/* construct full pathname to file */
@@ -60,21 +62,20 @@ recent_list_add(win_t *w, char *fname)
 	 * add doc to recent doc list, and add to recent doc menu.  note that
 	 * dnnp gets free'd down below, when trimming the recent doc list.
 	 */
-	dnnp = (doc_new_name_t *)g_malloc0(sizeof(doc_new_name_t));
+	dnnp = g_new0(doc_new_name_t, 1);
 	dnnp->w = w;
 	dnnp->fname = g_strdup(full);
 
-	/* if duplicate, remove it */
+	/* if duplicate, remove it from recentlist, and remove menu entry */
 	rlp = w->recentlist;
 	while (rlp) {
 		dnnp2 = (doc_new_name_t *)rlp->data;
 		if (strcmp(dnnp2->fname, dnnp->fname) == 0) {
 			w->recentlist = g_slist_remove_link(w->recentlist, rlp);
+			menu_recent_doc_remove(rlp->data);
+			g_slist_free_1(rlp);
 			g_free(dnnp2->fname);
 			g_free(dnnp2);
-#if GLIST_MAGIC
-			g_free(rlp);
-#endif
 			break;
 		}
 		rlp = rlp->next;
@@ -110,23 +111,23 @@ recent_list_init(win_t *w)
 		return;
 
 	if (recentrc == NULL) {
-		int len = strlen(prefs.appdir) + strlen("recentrc") + 1;
-		recentrc = (char *)g_malloc(len);
-		g_snprintf(recentrc, len, "%srecentrc", prefs.appdir);
+		int len = strlen(prefs.appdir) + strlen("recentrc") + 2;
+		recentrc = g_new(char, len);
+		g_snprintf(recentrc, len, "%s/recentrc", prefs.appdir);
 	}
 
 	if ((fp = fopen(recentrc, "r")) == NULL) {
 		/* not actually an error, since it's ok to not exist */
 		return;
 	}
-	buf = (char *)g_malloc(MAXPATH);
+	buf = g_new(char, MAXPATH);
 
 	ret = file_lock(recentrc, fp, FALSE, FALSE, TRUE);
 	while (fgets(buf, MAXPATH, fp)) {
 		if (buf[strlen(buf) - 1] == '\n')
 			buf[strlen(buf) - 1] = '\0';
 
-		dnnp = (doc_new_name_t *)g_malloc(sizeof(doc_new_name_t));
+		dnnp = g_new(doc_new_name_t, 1);
 		dnnp->w = w;
 		dnnp->fname = g_strdup(buf);
 		w->recentlist = g_slist_append(w->recentlist, dnnp);
@@ -163,7 +164,7 @@ recent_list_write(win_t *w)
 		return;
 
 	if ((fp = fopen(recentrc, "w")) == NULL) {
-		char *buf = (char *)g_malloc(strlen(recentrc) + 64);
+		char *buf = g_new(char, strlen(recentrc) + 64);
 		sprintf(buf, "recent_list_write: could not open "
 			"'%s'", recentrc);
 		perror(buf);
@@ -187,9 +188,28 @@ recent_list_write(win_t *w)
 } /* recent_list_write */
 
 
+void
+recent_list_free(win_t *w)
+{
+	doc_new_name_t *dnnp;
+	GSList *rlp;
+
+	while (w->recentlist) {
+		rlp = w->recentlist;
+		w->recentlist = g_slist_remove_link(w->recentlist, rlp);
+		dnnp = (doc_new_name_t *)(rlp->data);
+		g_slist_free_1(rlp);
+		g_free(dnnp->fname);
+		g_free(dnnp);
+	}
+} /* recent_list_free */
+
+
 /*** local function definitions ***/
 /*
  * PRIVATE: recent_list_trim
+ *
+ * trim list if we have too many entries
  */
 static void
 recent_list_trim(win_t *w, bool_t remove_from_menu)
@@ -197,7 +217,6 @@ recent_list_trim(win_t *w, bool_t remove_from_menu)
 	doc_new_name_t *dnnp;
 	GSList *rlp;
 
-	/* trim list if we have too many */
 	if (g_slist_length(w->recentlist) <= prefs.maxrecent)
 		return;
 
@@ -207,13 +226,12 @@ recent_list_trim(win_t *w, bool_t remove_from_menu)
 			menu_recent_doc_remove(rlp->data);
 		w->recentlist = g_slist_remove_link(w->recentlist, rlp);
 		dnnp = (doc_new_name_t *)(rlp->data);
+		g_slist_free_1(rlp);
 		g_free(dnnp->fname);
 		g_free(dnnp);
-#if GLIST_MAGIC
-		g_free(rlp);
-#endif
 	}
 } /* recent_list_trim */
 
+#endif	/* USE_RECENT */
 
 /* the end */
