@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: win.h,v 1.19 1999/01/18 20:08:51 kahn Exp kahn $
+ * $Id: win.h,v 1.35 2000/02/16 17:20:58 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -24,111 +24,102 @@
 extern "C" {
 #endif /* __cplusplus */
 
-
 #include <glib.h>
 #include <gtk/gtk.h>
 #include "main.h"
-
-/* search and replace options */
-#define REPLACE			0x00000001
-#define	CASE_SENSITIVE		0x00000002
-#define	PROMPT_REPLACE		0x00000004
-#define	REPLACE_ALL		0x00000008
-#define	REGULAR_EXPR		0x00000010
-
-#define DO_REPLACE(w)		(w->soptions & REPLACE)
-#define DO_CASE_SENSITIVE(w)	(w->soptions & CASE_SENSITIVE)
-#define DO_PROMPT_REPLACE(w)	(w->soptions & PROMPT_REPLACE)
-#define DO_REPLACE_ALL(w)	(w->soptions & REPLACE_ALL)
-#define DO_REGULAR_EXPR(w)	(w->soptions & REGULAR_EXPR)
-
-#define SET_REPLACE(w)		(w->soptions |= REPLACE)
-#define SET_CASE_SENSITIVE(w)	(w->soptions |= CASE_SENSITIVE)
-#define SET_PROMPT_REPLACE(w)	(w->soptions |= PROMPT_REPLACE)
-#define SET_REPLACE_ALL(w)	(w->soptions |= REPLACE_ALL)
-#define SET_REGULAR_EXPR(w)	(w->soptions |= REGULAR_EXPR)
-
-#define CLEAR_REPLACE(w)	(w->soptions &= ~REPLACE)
-#define CLEAR_CASE_SENSITIVE(w)	(w->soptions &= ~CASE_SENSITIVE)
-#define CLEAR_PROMPT_REPLACE(w)	(w->soptions &= ~PROMPT_REPLACE)
-#define CLEAR_REPLACE_ALL(w)	(w->soptions &= ~REPLACE_ALL)
-#define CLEAR_REGULAR_EXPR(w)	(w->soptions &= ~REGULAR_EXPR)
-
-
-#define MSGBAR_CLEAR	" "
-
+#include "search.h"
 
 /* the main window */
 typedef struct {
+#ifdef APP_DEBUG
 	unsigned	id;		/* window id */
+#endif
 	unsigned	rownum;		/* row # in window list popup */
 	GtkWidget *	toplev;		/* top-level window */
 	GtkWidget *	mainbox;	/* main box to hold all widgets */
 	GtkWidget *	menubar;	/* menus */
 	GtkWidget *	menubar_h;	/* menubar handle for floating menus */
+#ifdef USE_GNOME
+	GSList *	menuwgts;	/* toggle/radio buttons in menus */
+#else
+# ifdef GTK_HAVE_FEATURES_1_1_0
+	GtkItemFactory *mainfact;
+# else
+	GtkMenuFactory *mainfact;
+# endif
+#endif	/* USE_GNOME */
 #if defined(WANT_PROJECT) && defined(APP_GNP)
 	GtkWidget *	prjbar;		/* project status bar */
 #endif	/* WANT_PROJECT */
+#ifdef USE_TOOLBARS
+	GSList *	tblist;		/* list of all toolbars */
+	GSList *	tbhlist;	/* list of all toolbar handles */
 	GtkWidget *	main_tb;	/* toolbar */
-	GtkWidget *	main_tb_h;	/* toolbar handle */
-#ifdef APP_GNP
-	GtkWidget *	html_tb;	/* html toolbar */
+# ifdef USE_HTMLTAGS
+	GSList *	html_tblist;	/* list of HTML related toolbars */
 	GtkWidget *	html_tb_h;	/* html toolbar handle */
-#endif	/* APP_GNP */
+	GtkWidget *	html_dlg;	/* for HTML tag insertion dialogs */
+	GtkWidget *	tagchooser;	/* top-level tag chooser window */
+	GtkWidget *	tc_nb;		/* tag chooser's notebook */
+	int		tc_row;		/* tag chooser's selected row */
+# endif
+#endif	/* USE_TOOLBARS */
 	GtkWidget *	nb;		/* notebook which contains docs */
+#ifndef USE_GNOME
 	GtkWidget *	hbox_bot;	/* holds msgbar and docinfo button */
+#endif
 	GtkWidget *	msgbar;		/* message bar */
+#ifdef USE_FILEINFO
 	GtkWidget *	docinfo;	/* docinfo button */
 	GtkWidget *	docinfo_label;	/* docinfo label  */
+#endif
 	char *		lastmsg;	/* last message shown */
 	int		timeout_id;	/* msgbar timeout id */
 	GtkWidget *	quickmenu;	/* popup menu on right mouse click */
+#ifdef USE_RANDOMTIPS
+	int		rtips_timeout;	/* random tips timeout id */
+#endif	/* USE_RANDOMTIPS */
 
 	/* document related info.  note that we alreays track the current
 	   document in curdoc whenever a doc is opened/closed.  this is to
-	   avoid constantly traversing the doclist */
+	   avoid constantly traversing the doclist, like another gtk/gnome
+	   editor does in a braindead fashion */
 	GSList *	doclist;	/* doc_t list */
 	gpointer	curdoc;		/* ptr to doclist->data (doc_t *) */
 	guint		numdoc;		/* number of docs */
 #ifdef APP_GNP
 	bool_t		allclosed;	/* indicate all docs were closed */
 #endif	/* APP_GNP */
+#ifdef USE_RECENT
 	GSList *	recentlist;	/* this window's recent doc list */
+#endif	/* USE_RECENT */
 
 	/* file selection dialog boxes; one per window */
-#ifdef APP_GNP
 	GtkWidget *	saveas;		/* save-as file selector */
-#endif	/* APP_GNP */
 	GtkWidget *	filesel;	/* open file selector */
 
+#ifdef USE_DOCLIST
 	/* the following are used for the doc list popup window */
 	GtkWidget *	dlw;		/* doc list window */
 	GtkWidget *	dlw_data;	/* doc list window data */
-	GtkWidget *	dlw_tb;		/* doc list window toolbar */
-	GtkWidget *	dlw_tb_h;	/* doc list window toolbar handle */
-
+#endif	/* USE_DOCLIST */
 #if defined(WANT_PROJECT) && defined(APP_GNP)
-	/* project information */
-	void *		prj;
+	GSList *	prjlist;	/* projects list */
+	GtkWidget *	prjsel;		/* open project selector */
+	GtkWidget *	prjlw;		/* projects list window */
+	GtkWidget *	prjlw_data;	/* projects list window data */
 #endif	/* WANT_PROJECT */
-
-	/* search and replace information */
-	GtkWidget *	search;		/* top level dialog widget */
-	GtkWidget *	stxt_wgt;	/* search text entry widget */
-	GtkWidget *	rtxt_wgt;	/* replace text entry widget */
-	GtkWidget *	startcur;	/* start at cur pos radio button */
-	GtkWidget *	startbeg;	/* start at doc beginning radio btn */
-	GtkWidget *	casesens;	/* case-sensitive toggle button */
-#ifdef NOTYET
-	GtkWidget *	regexp;		/* regular expr toggle button */
-#endif
-	char *          stxt;		/* search text for 'search again' */
-	char *          rtxt;		/* replace text */
-	long		soptions;	/* search options */
+#ifdef USE_SEARCH
+	search_t	*srg;		/* search/replace and goto line info */
+#endif	/* USE_SEARCH */
 } win_t;
 
 
+#ifdef USE_AUTOSAVE
 extern int	win_autosave(gpointer cbdata);
+extern void	win_autosave_init(void);
+extern void	win_autosave_reset(void *);
+#endif	/* USE_AUTOSAVE */
 extern void	win_close_all_cb(GtkWidget *wgt, gpointer cbdata);
 extern void	win_close_cb(GtkWidget *wgt, gpointer cbdata);
 extern void	win_doc_foreach(void (*func)(void *));
@@ -141,13 +132,18 @@ extern void	win_dtab_top(GtkWidget *wgt, gpointer cbdata);
 extern void	win_dtab_bot(GtkWidget *wgt, gpointer cbdata);
 extern void	win_dtab_left(GtkWidget *wgt, gpointer cbdata);
 extern void	win_dtab_right(GtkWidget *wgt, gpointer cbdata);
-extern void	win_list_set_curdoc(win_t *w);
-extern void	win_list_set_numdoc(win_t *w);
 extern void	win_redraw_doc_tab(void *);
 #ifdef APP_GNP
 extern void	win_wwrap_toggle(GtkWidget *wgt, gpointer cbdata);
 #endif	/* APP_GNP */
 extern void	win_doc_reset_style(gpointer data);
+#ifdef USE_WINLIST
+extern void	win_list_set_curdoc(win_t *w);
+extern void	win_list_set_numdoc(win_t *w);
+#else
+# define	win_list_set_curdoc(w)
+# define	win_list_set_numdoc(w)
+#endif
 
 
 #ifdef __cplusplus
